import cartopy.io.shapereader as shpreader
import sys, os
import json
import collections
from datetime import datetime, timedelta

pltg = os.environ["PLTG"]

bopacity = "100"
opacity = "0"
color = bcolor = "255 0 0"
width = 0.5

indir = outdir = f"{pltg}/lightningcast/static/shapefiles/"

offshore_reader = shpreader.Reader(f"{indir}/oz16ap19.shp")
recs = list(offshore_reader.records())

# starting the json
outdict = collections.OrderedDict()
outdict["type"] = "FeatureCollection"
# feature list
flist = []

# skip non-OPC/TAFB offshore regions
delete = []
for ii, rec in enumerate(recs):
    if (
        "Hawaii" in rec.attributes["Name"]
        or "Alaska" in rec.attributes["Name"]
        or "Bering" in rec.attributes["Name"]
        or "Arctic" in rec.attributes["Name"]
    ):
        continue
    atts = rec.attributes
    poly = rec.geometry

    simp = poly.simplify(0.05, preserve_topology=False)

    try:
        coords = [
            [
                [x, y]
                for x, y in zip(simp.exterior.coords.xy[0], simp.exterior.coords.xy[1])
            ]
        ]
    except:
        for geom in simp.geoms:
            coords = [
                [
                    [x, y]
                    for x, y in zip(
                        geom.exterior.coords.xy[0], geom.exterior.coords.xy[1]
                    )
                ]
            ]
            break  # We're assuming the first geom is the "main" polygon

    fdict = collections.OrderedDict({"type": "Feature"})
    fdict["geometry"] = collections.OrderedDict(
        {"type": "Polygon", "coordinates": coords}
    )

    fdict["properties"] = {
        "INFO": "",
        "BCOLOR": bcolor,
        "COLOR": color,
        "WIDTH": width,
        "OPACITY": opacity,
        "BOPACITY": bopacity,
    }
    for key, val in atts.items():
        if val == "":
            continue
        fdict["properties"]["INFO"] += f"{key}: {val}<br>"
    flist.append(fdict)


outdict["features"] = flist
outjson_obj = json.dumps(outdict, indent=None)

now = datetime.utcnow()

of = open(now.strftime(f"{outdir}/OFFSHOREZONES_%Y%m%d_%H%M00.force.json"), "w")
of.write(outjson_obj)
of.close()
