import numpy as np
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
import os, sys
from lightningcast import utils

abi_area_def, _, _, projx, projy = utils.get_area_definition(
    "static/GOES_East.nc", return_xy=True
)

fig = plt.figure(figsize=(10, 7))
crs = abi_area_def.to_cartopy_crs()
ax = fig.add_axes([0, 0, 1, 1], projection=crs)
ax.coastlines(color="black", linewidth=2.0)
ax.set_extent([projx.min(), projx.max(), projy.min(), projy.max()], crs=crs)

ny, nx = (480, 480)

Xs = np.arange(50, 2500 - nx, nx, dtype=int)
Ys = np.arange(50, 1500 - ny, ny, dtype=int)

for X in Xs:
    for Y in Ys:
        ax.plot(
            [projx[X], projx[X + nx], projx[X + nx], projx[X], projx[X]],
            [projy[Y], projy[Y], projy[Y + ny], projy[Y + ny], projy[Y]],
            marker=None,
            color="red",
            linewidth=4,
            transform=crs,
        )

plt.savefig("boxes.png", bbox_inches="tight")
