import zarr
import numpy as np
import os, sys
from datetime import datetime, timedelta, timezone

ds = zarr.open("/home/jcintineo/pltgALL.zarr", "r")

abi_date = ds.ABI_date[:]
abi_time = ds.ABI_time[:]

epoch0 = datetime(1970, 1, 1, 0, 0)

dts = np.array(
    [
        datetime.strptime(str(dd) + str(tt).zfill(4), "%Y%m%d%H%M")
        for dd, tt in zip(abi_date, abi_time)
    ]
)

chunkind = np.arange(0, ds.CH13.shape[0], 20000)

# trueind = np.arange(ds.CH13.shape[0])

for ll in range(len(chunkind)):
    print(chunkind[ll])
    try:
        ch13 = ds.CH13[chunkind[ll] : chunkind[ll + 1]]
        ch02 = ds.CH02[chunkind[ll] : chunkind[ll + 1]]
        ch05 = ds.CH05[chunkind[ll] : chunkind[ll + 1]]
        ch06 = ds.CH06[chunkind[ll] : chunkind[ll + 1]]
        lat = ds.lat[chunkind[ll] : chunkind[ll + 1]]
    except IndexError:
        ch13 = ds.CH13[chunkind[ll] :]
        ch02 = ds.CH02[chunkind[ll] :]
        ch05 = ds.CH05[chunkind[ll] :]
        ch06 = ds.CH06[chunkind[ll] :]
        lat = ds.lat[chunkind[ll] :]

    uniq = np.array([])
    ind0 = np.where(np.abs(ch02) > 2)
    if len(ind0[0]):
        uniq = np.concatenate((uniq, np.unique(ind0[0])))
    ind1 = np.where(np.abs(ch05) > 2)
    if len(ind1[0]):
        uniq = np.concatenate((uniq, np.unique(ind1[0])))
    ind2 = np.where(np.abs(ch06) > 2)
    if len(ind2[0]):
        uniq = np.concatenate((uniq, np.unique(ind2[0])))
    ind3 = np.where(np.abs(ch13) > 500)
    if len(ind3[0]):
        uniq = np.concatenate((uniq, np.unique(ind3[0])))
    ind4 = np.where(lat < 27.5)
    if len(ind4[0]):
        uniq = np.concatenate(
            (uniq, np.unique(ind4[0]))
        )  # eliminating any potential "Bermuda strobes"

    new_data = {}
    if len(uniq):
        uniq = np.unique(uniq)

        # now remove bad samples
        for key in ds.keys():
            try:
                new_data[key] = np.delete(
                    ds[key][chunkind[ll] : chunkind[ll + 1]], uniq, axis=0
                )
            except IndexError:
                new_data[key] = np.delete(ds[key][chunkind[ll] :], uniq, axis=0)

        print("deleting", len(uniq), "elements")

    else:  # just copy data
        for key in ds.keys():
            try:
                new_data[key] = ds[key][chunkind[ll] : chunkind[ll + 1]]
            except IndexError:
                new_data[key] = ds[key][chunkind[ll] :]

    # now write out new zarr
    # write zarr
    zname = "/home/jcintineo/pltgALL_new.zarr"
    ds_exists = True if (os.path.isdir(zname)) else False
    if not (ds_exists):
        root = zarr.open(zname, mode="a")
    chunksize = 300

    for key in ds.keys():
        # set chunksizes
        if len(ds[key].shape) == 1:
            chunks = (chunksize,)
        else:
            chunks = (chunksize, ds[key].shape[1], ds[key].shape[2], 1)

        # create data
        if ds_exists:  # just append along time axis
            newarr = zarr.zeros(
                new_data[key].shape, chunks=chunks, dtype=new_data[key].dtype
            )
            newarr[:] = new_data[key]
            root[key].append(newarr, axis=0)
        else:  # start of new data array in group
            newarr = root.zeros(
                key, shape=new_data[key].shape, chunks=chunks, dtype=new_data[key].dtype
            )
            newarr[:] = new_data[key]
            # set attributes
            for att in ds[key].attrs:
                newarr.attrs[att] = ds[key].attrs[att]

    print("-----")
