import zarr
import numpy as np
import pandas as pd
import xarray as xr
from glob import glob
import os, sys
from datetime import datetime, timedelta
import time
from multiprocessing import Pool
from netCDF4 import Dataset
from collections import OrderedDict
import itertools

os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
sys.path.insert(0, "..")
from solar_angles import solar_angles

t0 = time.time()


def get_data(ncs, seq_len):

    recs = {
        "ABI_date": [],
        "ABI_time": [],
        "CH02": [],
        "CH05": [],
        "CH06": [],
        "CH08": [],
        "CH10": [],
        "CH11": [],
        "CH13": [],
        "CH15": [],
        "CH16": [],
        "FED": [],
        "MAXFED_5to60": [],
        "MAXFED_5to30": [],
        "MAXFED_5to45": [],
        "ENI": [],
        "MAXENI_0to60": [],
        "MAXENI_0to45": [],
        "MAXENI_0to30": [],
        "u925": [],
        "v925": [],
        "u850": [],
        "v850": [],
        "u700": [],
        "v700": [],
        "u500": [],
        "v500": [],
        "GHRSST": [],
    }
    atts = ["ABI_date", "ABI_time"]

    for s in ncs:
        variables = {
            "CH02": [],
            "CH05": [],
            "CH06": [],
            "CH08": [],
            "CH10": [],
            "CH11": [],
            "CH13": [],
            "CH15": [],
            "CH16": [],
            "FED": [],
            "MAXFED_5to60": [],
            "MAXFED_5to45": [],
            "MAXFED_5to30": [],
            "ENI": [],
            "MAXENI_0to60": [],
            "MAXENI_0to45": [],
            "MAXENI_0to30": [],
            "u925": [],
            "v925": [],
            "u850": [],
            "v850": [],
            "u700": [],
            "v700": [],
            "u500": [],
            "v500": [],
            "GHRSST": [],
        }

        print(s)
        # goes16_conus_20180611-001719.netcdf
        # find previous seq_len times
        basefile = os.path.basename(s)
        dirname = os.path.dirname(s)
        dirbase = os.path.dirname(dirname)  # remove yyyymmdd
        parts = basefile.split("_")
        dt = datetime.strptime(parts[-1][0:15], "%Y%m%d-%H%M%S")
        prev_dts = [dt - timedelta(minutes=sl * 5) for sl in range(seq_len)]
        prev_dts.reverse()

        cont = False
        for pp in range(len(prev_dts)):
            try:
                pnc = glob(
                    prev_dts[pp].strftime(
                        os.path.join(
                            dirbase,
                            "%Y%m%d",
                            ("_").join([parts[0], parts[1], "%Y%m%d-%H%M*.netcdf"]),
                        )
                    )
                )[0]
            except IndexError:
                cont = True
                break
            else:
                nc = Dataset(pnc, "r")
                nc.set_auto_mask(False)

                # if(pp == 0): #test
                #  #give tmp = 0 a 15% chance of going through
                #  tmp = 1 if(np.max([nc.maxFED_t10,nc.maxFED_t15,nc.maxFED_t20,nc.maxFED_t25,nc.maxFED_t30]) >= 1) else 0
                #  if(tmp == 0):
                #    if(np.random.random(1)[0] < 0.85):
                #      cont = True
                #      nc.close()
                #      break
                if pp == (seq_len - 1):
                    # still need to add the last frame to vars here
                    for var in variables:
                        # QC if/elif blocks
                        if var in ["CH02", "CH05", "CH06"]:
                            if np.max(np.abs(nc.variables[var][:])) > 2:
                                cont = True
                                break
                        elif var == [
                            "CH08",
                            "CH09",
                            "CH10",
                            "CH11",
                            "CH12",
                            "CH13",
                            "CH14",
                            "CH15",
                            "CH16",
                        ]:
                            if np.max(np.abs(nc.variables[var][:])) > 500:
                                cont = True
                                break

                        variables[var].append(nc.variables[var][:])

                    # put atts after vars so that QCing of vars will short circuit this loop and we won't add atts to recs
                    if cont:
                        pass  # this just passes on to the next step...which should be closing the netcdf, then
                        # moving on to the next sample
                    else:
                        for att in atts:
                            if att == "ABI_date":
                                val = int(dt.strftime("%Y%m%d"))
                            elif att == "ABI_time":
                                val = int(dt.strftime("%H%M"))
                            else:
                                val = nc.getncattr(att)
                            recs[att].append(val)

                        # else:
                        #  for var in variables:
                        #    #QC if/elif blocks
                        #    if(var in ['CH02','CH05','CH06']):
                        #      if(np.max(np.abs(nc.variables[var][:])) > 2):
                        #        cont = True
                        #        break
                        #    elif(var == 'CH13'):
                        #      if(np.max(np.abs(nc.variables[var][:])) > 500):
                        #        cont = True
                        #        break

                        variables[var].append(nc.variables[var][:])

                nc.close()

        if cont:
            continue  # insufficient number of previous frames

        for var in variables:
            variables[var] = np.stack(variables[var], axis=-1)
            recs[var].append(variables[var])

    return recs


##################################################################################################################

if __name__ == "__main__":
    date = sys.argv[1]

    nthreads = 12
    chunksize = 1  # number of images/samples to chunk or read at once
    seq_len = 1

    ncdir = "/apollo/grain/jcintineo/AI/OCONUS/seg/netcdf400x400_eni_hrrr_ghrsst/"
    zname = "/apollo/grain/jcintineo/AI/OCONUS/seg/unet_train400_5frames_eni_hrrr_ghrsst.zarr"

    for hh in [
        "0[0-4]",
        "0[5-9]",
        "1[0-4]",
        "1[5-9]",
        "2",
    ]:  # this is to keep the RAM down
        pool = Pool(nthreads)

        allncs = np.sort(
            glob(
                os.path.join(
                    ncdir, date, "goes16_conus_" + date + "-" + hh + "*.netcdf*"
                )
            )
        )

        if len(allncs) == 0:
            continue

        # perform shuffling here
        # shuff_ind = np.random.permutation(len(allncs))
        # allncs = allncs[shuff_ind]

        interval = int(np.ceil(len(allncs) / nthreads))

        list_of_lists = []

        for n in range(nthreads):
            list_of_lists.append(allncs[n * interval : n * interval + interval])

        results = pool.starmap(get_data, zip(list_of_lists, itertools.repeat(seq_len)))
        pool.close()
        pool.join()

        fields = OrderedDict()
        fieldnames = [
            "ABI_date",
            "ABI_time",
            "CH02",
            "CH05",
            "CH06",
            "CH08",
            "CH10",
            "CH11",
            "CH13",
            "CH15",
            "CH16",
            "FED",
            "MAXFED_5to60",
            "MAXFED_5to45",
            "MAXFED_5to30",
            "ENI",
            "MAXENI_0to60",
            "MAXENI_0to45",
            "MAXENI_0to30",
            "mlcape",
            "u925",
            "v925",
            "u850",
            "v850",
            "u700",
            "v700",
            "u500",
            "v500",
            "GHRSST",
        ]
        for f in fieldnames:
            fields[f] = []

        datasets = {
            "CH02": {
                "atts": {
                    "desc": "0.65-micron refl from ABI",
                    "units": "dimensionless",
                    "mean": 0.087134,
                    "std": 0.15314,
                }
            },
            "CH05": {
                "atts": {
                    "desc": "1.6-micron refl from ABI",
                    "units": "dimensionless",
                    "mean": 0.047332,
                    "std": 0.0834376,
                }
            },
            "CH06": {
                "atts": {
                    "desc": "2.26-micron refl from ABI",
                    "units": "dimensionless",
                    "mean": 0.04413,
                    "std": 0.080274,
                }
            },
            "CH08": {
                "atts": {
                    "desc": "6.2-micron BT from ABI",
                    "units": "K",
                    "mean": 234.73875,
                    "std": 6.4855,
                }
            },
            "CH10": {
                "atts": {
                    "desc": "7.3-micron BT from ABI",
                    "units": "K",
                    "mean": 252.71758,
                    "std": 11.06057,
                }
            },
            "CH11": {
                "atts": {
                    "desc": "8.2-micron BT from ABI",
                    "units": "K",
                    "mean": 274.36105,
                    "std": 21.69784,
                }
            },
            "CH13": {
                "atts": {
                    "desc": "10.3-micron BT from ABI",
                    "units": "K",
                    "mean": 270.930,
                    "std": 25.9126,
                }
            },
            "CH15": {
                "atts": {
                    "desc": "12.4-micron BT from ABI",
                    "units": "K",
                    "mean": 273.168,
                    "std": 19.9895,
                }
            },
            "CH16": {
                "atts": {
                    "desc": "13.3-micron BT from ABI",
                    "units": "K",
                    "mean": 260.5381,
                    "std": 17.2468,
                }
            },
            "FED": {
                "atts": {
                    "desc": "GLM flash extent density, accumulated over 5 min",
                    "units": "fl",
                    "mean": 0.3536,
                    "std": 2.341,
                }
            },
            "MAXFED_5to60": {"atts": {}},
            "MAXFED_5to45": {"atts": {}},
            "MAXFED_5to30": {"atts": {}},
            "ENI": {"atts": {"desc": "ENI flash density", "units": "fl/min/km^2"}},
            "MAXENI_0to60": {"atts": {}},
            "MAXENI_0to45": {"atts": {}},
            "MAXENI_0to30": {"atts": {}},
            "mlcape": {"atts": {}},
            "u925": {"atts": {}},
            "v925": {"atts": {}},
            "u850": {"atts": {}},
            "v850": {"atts": {}},
            "u700": {"atts": {}},
            "v700": {"atts": {}},
            "u500": {"atts": {}},
            "v500": {"atts": {}},
            "GHRSST": {"atts": {}},
            "ABI_date": {"atts": {}},
            "ABI_time": {"atts": {}},
        }

        # write zarr
        ds_exists = True if (os.path.isdir(zname)) else False
        root = zarr.open(zname, mode="a")

        for f in fields:
            # if(f == 'solzen'):
            #  dts = [datetime.strptime(str(x)+str(y).zfill(4),'%Y%m%d%H%M') for x,y in zip(fields['ABI_date'],fields['ABI_time'])]
            #  solzen,solaz = solar_angles(dt=dts,lon=fields['lon'],lat=fields['lat'])
            #  fields[f] = solzen
            # else:
            for ii in range(
                len(results)
            ):  # concatenate list of results (dicts) and then stck them into an array
                fields[f] += results[ii][f]
            fields[f] = np.stack(fields[f])
            if f == "FED":
                ind = fields[f] > 9999
                fields[f][ind] = 0
            datasets[f]["data"] = fields[f]

            # set chunksizes
            if len(datasets[f]["data"].shape) == 1:
                chunks = (10000,)
            elif (
                f.startswith("MAXFED")
                or f.startswith("MAXENI")
                or f.startswith("u")
                or f.startswith("v")
                or f == "mlcape"
                or f == "GHRSST"
            ):
                chunks = (
                    chunksize,
                    datasets[f]["data"].shape[1],
                    datasets[f]["data"].shape[2],
                    1,
                )
            else:
                chunks = (
                    chunksize,
                    datasets[f]["data"].shape[1],
                    datasets[f]["data"].shape[2],
                    datasets[f]["data"].shape[3],
                    1,
                )

            # create data
            if ds_exists:  # just append along time axis
                newarr = zarr.zeros(
                    datasets[f]["data"].shape,
                    chunks=chunks,
                    dtype=str(datasets[f]["data"].dtype),
                )
                newarr[:] = datasets[f]["data"]
                root[f].append(newarr, axis=0)
            else:  # start of new data array in group
                newarr = root.zeros(
                    f,
                    shape=datasets[f]["data"].shape,
                    chunks=chunks,
                    dtype=str(datasets[f]["data"].dtype),
                )
                newarr[:] = datasets[f]["data"]
                # set attributes
                for att in datasets[f]["atts"]:
                    newarr.attrs[att] = datasets[f]["atts"][att]

    print(time.time() - t0)
